15 => int numSpaces;
3 => int numInstruments;

1 => float lightIntensity;
GG.scene().light().ambient(@(lightIntensity,lightIntensity,lightIntensity));
GG.scene().light().diffuse(@(lightIntensity,lightIntensity,lightIntensity));
GG.camera() --> GGen dolly --> GG.scene(); // put camera on a dolly
GG.camera().pos( @(8, 8, 30) ); // position

//setup
Board board --> GG.scene();
Player1 p1 --> GG.scene();
// Player2 p2 --> board; // Player2 p2 --> GG.scene();
SndBuf p1Ring => dac;
// SndBuf p2Ring => dac;
0.1 => p1Ring.gain;
// 0.03 => p2Ring.gain;
me.dir() + "hurt.wav" => p1Ring.read;
// me.dir() + "ring2.wav" => p2Ring.read;

//plays the "hurt" sound
fun void audioP1() {
    p1Ring.samples() / 10 => p1Ring.pos;
    80::ms => now;
}

// fun void audioP2() {
//     p2Ring.samples() / 10 => p2Ring.pos;
//     80::ms => now;
// }

//back when there were two players, this would allow one player to eat the other's pellets...
fun void checkIntersectionP1() {
    p1.posAsKey() => string key;
    board.getIndex(key) => int ind;
    if (board.hasFood(ind) && board.getFoodType(ind) == 1) {
        //get food type
        
        board.removeFood(ind);
        //change sound of player 1? play a sound?
    }
}

//put down a pellet at the player's current location
fun void placeFood(int type) {
    string key;
    p1.posAsKey() => key;
    board.getIndex(key) => int ind;
    board.placeFood(ind, type);
}


/* * * * *
 * SEQUENCER INFO/INSTRUMENTS
 * * * * */
120 => int BPM;  // beats per minute
(1.0/BPM)::minute / 2.0 => dur step;  // step duration

[
    3, 5, 7, 10, 12, 15, 18, 21, 24, 27, 30, 33, 36, 39, 42, 45, 48
] @=> int scale[];  // relative MIDI offsets for minor pentatonic scale; from Andrew's code!

// base class for percussion instruments
class Instrument extends Chugraph {
    fun void play() {}
}

2::second => dur duration;

/* * *
 * thank you clint hoagland one of my best friends in this class (jk)
 * https://www.youtube.com/watch?v=JlikA_zDbnc
 * * */
//noise osc; background tch sound; percussion!
class Noiser extends Instrument {
    //added JCRev
    Noise oscNoise => ADSR envNoise => LPF lpfNoise => JCRev revNoise => dac;
    0.1 => oscNoise.gain;
    36 => int offset;
    (44100 / 16) * 4 => lpfNoise.freq;
    (1::ms, duration / 8, 0.0, 1::ms) => envNoise.set;
    duration => dur durNoise;
    0.0 => revNoise.mix;

    fun void setMix(float mix, int bits) {
        mix => revNoise.mix;
    }

    fun void randomizeDur() {
        Math.random2(0, 1) => int divider;
        if (divider) {
            duration / 8.0 => durNoise;
        } else {
            duration => durNoise;
        }
    }
    fun void setFreq(float freq) {
        (44100 / 16) * Math.log(freq)  => lpfNoise.freq;
    }

    fun void play() {
        randomizeDur();
        (1::ms, durNoise, 0.0, 1::ms) => envNoise.set;
        1 => envNoise.keyOn;
        50::ms => now;
        1 => envNoise.keyOff;
        // e.releaseTime() => now;
    }
}

//pulse; static-y blare kind of. increase width to make it sound clearer/less shrill
class Pulser extends Instrument { 
    PulseOsc oscPulse => ADSR envPulse => Bitcrusher bc => JCRev revPulse => dac;
    0.1 => oscPulse.gain;
    48 => int offsetPulse;
    0.125 => oscPulse.width; //0.125 or 0.25; randomize?
    (1::ms, duration, 0, 1::ms) => envPulse.set;
    0.0 => revPulse.mix;
    0.1 => bc.gain;

    fun void setMix(float mix, int bits) {
        mix => revPulse.mix;
        bits => bc.bits;
        bits => bc.downsampleFactor;
    }

    fun void randomizeWidth() {
        Math.random2(0, 1) => int divider;
        if (divider) {
            0.125 => oscPulse.width;
        } else {
            0.25 => oscPulse.width;
        }
    }

    fun void setFreq(float freq) {
        Std.mtof(offsetPulse) + freq => oscPulse.freq;
    }

    fun void play() {
        randomizeWidth();
        1 => envPulse.keyOn;
        50::ms => now;
        1 => envPulse.keyOff;
        // e.releaseTime() => now;
    }
}

//triosc; clean sound. middle
class Trier extends Instrument { 
    TriOsc oscTri => ADSR envTri => Bitcrusher bc => JCRev revTri => dac;
    0.2 => oscTri.gain;
    36 => int offsetTri;
    (1::ms, duration, 0, 1::ms) => envTri.set;
    0.0 => revTri.mix;
    0.2 => bc.gain;

    fun void setMix(float mix, int bits) {
        mix => revTri.mix;
        bits => bc.bits;
        bits => bc.downsampleFactor;
    }

    fun void setFreq(float freq) {
        Std.mtof(offsetTri) + freq => oscTri.freq;
    }

    fun void play() {
        1 => envTri.keyOn;
        50::ms => now;
        1 => envTri.keyOff;
        // e.releaseTime() => now;
    }
}

Noiser noise;
Pulser pulse;
Trier tri;

60 - 2 * 12 => int root;

int canEat[numInstruments]; //can it eat the player?
int foodEaten[numInstruments]; //num food each playhead has eaten per "run"

0 => int playheadCol;
0 => int numRounds;
0 => int wonkyCounter;
0 => int wonky;
while( true ) {
    playheadCol % numSpaces => playheadCol;

    if (numRounds % (numSpaces * numInstruments) == 0) {
        0 => foodEaten[0];
        0 => foodEaten[1];
        0 => foodEaten[2];
    }

    // checkIntersectionP1(); //check if there's food interaction

    //check/update what can be eaten
    for (0 => int i; i < numInstruments; i++) {
        board.findFoodInArea(i) @=> int foodLoc[];
        if (foodLoc[0] > 0) { //go eat food first!
            0 => canEat[i];
            board.isAngry(i, 0);
        } else { //playhead can eat the player...
            //change playhead color
            1 => canEat[i];
            board.isAngry(i, 1);
        }
    }

    board.playheadAnim(numRounds);

    
    board.setPlayhead((playheadCol + 1) % numSpaces); //playhead action

    if ( KB.isKeyDown(KB.KEY_SPACE) ) {
        placeFood(2);
    }

    //check if there's food/players at playhead
    board.playheadToInds(playheadCol) @=> int playheadInds[];
    board.getY(playheadCol) => float currY; //column

    //check if playhead-food intersection
    for(int ind: playheadInds) {
        (ind - 1 ) / numSpaces => int row; //which row
        row / (numSpaces / numInstruments) => int instrument; //which instrument

        // if (canEat(instrument)) {
        //     board.playheadAnim(numRounds, 1);
        // }

        if (board.hasFood(ind)) {
            if (wonky && wonkyCounter > numSpaces) { //fix all the mixes
                0 => wonky;
                0 => wonkyCounter;
                noise.setMix(0.0, 1);
                tri.setMix(0.0, 1);
                pulse.setMix(0.0, 1);
                p1.isFine();
            }
            
            Std.mtof(root + scale[ind / numSpaces]) => float currFreq;
            
            //play sound
            if (instrument == 0) { // noise
                noise.setFreq(currFreq);
                spork ~ noise.play();
            } else if (instrument == 1) { //tri
                tri.setFreq(currFreq);
                spork ~ tri.play();
            } else if (instrument == 2) { //pulse
                pulse.setFreq(currFreq);
                spork ~ pulse.play();
            }

            if (foodEaten[instrument] < 1) { //eat it! once per time period
                board.removeFood(ind);
                foodEaten[instrument] + 1 => foodEaten[instrument];
            }
        } else if (canEat[instrument] == 1 && p1.isAt(currY, board.getX(row))) {
            noise.setMix(1.0, 32);
            tri.setMix(1.0, 32);
            pulse.setMix(1.0, 32);
            1 => wonky;
            spork ~ audioP1(); //ouch!
            p1.isHurt();
        }
    }
    step => now;
    playheadCol++;
    numRounds++;
    wonkyCounter++;
    GG.nextFrame() => now; // next graphics frame
}